module SSDP;

export {
	redef enum Log::ID += { SSDP_LOG };

    ## Default logging policy hook for SSDP_LOG.
    global log_policy: Log::PolicyHook;

	type Info: record {
		## Time
		ts: time &log &default=network_time();

		## Unique ID for the connection
		uid: string &log;

		## The connection's 4-tuple of endpoint addresses/ports
		id: conn_id &log;

		## Is orig side of connection
		is_orig: bool &log;

		## The operation
		operation: string &log &optional;

		## The host
		host: string &log &optional;

		## The USN
		usn: string &log &optional;

		## The target (notification or search)
		target: string &log &optional;

		## The target sub type
		target_sub: string &log &optional;

		## The location
		location: string &log &optional;

		## The user agent
		user_agent: string &log &optional;

		## The server info
		server: string &log &optional;

		## The Man field
		man: string &log &optional;

		## The MX field
		mx: string &log &optional;

		## The CACHE-CONTROL field
		cache_control: string &log &optional;

		## Remainder of header names
		remaining_header_names: vector of string &log &optional;

		## Remainder of header values
		remaining_header_values: vector of string &log &optional;
	};

	# Set to true if you want an ssdp.log.  Caution!!! SSDP is chatty and could create a lot of logs!
	option enable_logging: bool = T;

	### Events ###

	# Raised for every SSDP message
	global SSDP::message: event(c: connection, is_orig: bool, operation: string);

	# Raised for every SSDP header
	global SSDP::header: event(c: connection, is_orig: bool, name: string, value: string);

	# Log event
	global SSDP::log_ssdp: event(rec: SSDP::Info);

}

redef record connection += {
	ssdp: Info &optional;
};

function new_info(c: connection)
	{
	c$ssdp = [$uid=c$uid, $id=c$id, $is_orig=T];
	c$ssdp$remaining_header_names = vector();
	c$ssdp$remaining_header_values = vector();
	}

function set_session(c: connection)
	{
	if ( ! c?$ssdp )
		{
		new_info(c);
		}
	}

event SSDP::message(c: connection, is_orig: bool, operation: string)
	{
	if ( ! c?$ssdp )
		return;
	
	c$ssdp$is_orig = is_orig;
	c$ssdp$operation = operation;

	if ( enable_logging )
		Log::write(SSDP::SSDP_LOG, c$ssdp);

	# Clear out data for our next message
	new_info(c);
	}

event SSDP::header(c: connection, is_orig: bool, name: string, value: string)
	{
	set_session(c);

	local lil_name = to_lower(name);

	switch(lil_name) 
		{
		case "user-agent":
			c$ssdp$user_agent = value;
			break;
		case "nt":
			fallthrough;
		case "st":
			c$ssdp$target = value;
			break;
		case "nts":
			c$ssdp$target_sub = value;
			break;
		case "location":
			c$ssdp$location = value;
			break;
		case "usn":
			c$ssdp$usn = value;
			break;
		case "server":
			c$ssdp$server = value;
			break;
		case "host":
			c$ssdp$host = value;
			break;
		case "man":
			c$ssdp$man = value;
			break;
		case "mx":
			c$ssdp$mx = value;
			break;
		case "cache-control":
			c$ssdp$cache_control = value;
			break;
		default:
			c$ssdp$remaining_header_names += name;
			c$ssdp$remaining_header_values += value;
			break;
		}
	}

event zeek_init() &priority=5 
	{
	Log::create_stream(SSDP::SSDP_LOG, [$columns=Info, $ev=log_ssdp, $path="ssdp", $policy=log_policy]);
	}
